/*
 Standalone Current Logger for ATtiny84
 Version: 0.3
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 25/04/2012
 Last Modified: 29/05/2012

 */

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void system_sleep(void) {
	cbi(ADCSRA,ADEN); // Switch Analog to Digital converter OFF
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
	sbi(ADCSRA,ADEN);  // Switch Analog to Digital converter ON
}

// Used and modified from Arduino wiring.c - to setup the ATtiny
void setup(void) {
	
	// Change 2MHz to 4 MHz by changing clock prescaler to 4
	CLKPR = (1<<CLKPCE); // Prescaler enable
	CLKPR = (1<<CLKPS1); // Clock division factor 4 (0010)
	
	sei(); // Turn on interrupts
	
	// Set ADC prescale factor to 32. (4 MHz clock / 32 = 125 KHz, inside the desired 50-200 KHz range)
	sbi(ADCSRA, ADPS2);
	sbi(ADCSRA, ADPS0);
	
	ADMUX = (1 << REFS1) | (adcPin & 0x3f);
}

// Begin V-USB default required code
static uchar reportBuffer[2];    /* buffer for HID reports */
static uchar idleRate;           /* in 4 ms units */
static uchar valueBuffer[20];
static uchar *nextDigit;

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */

/* Keyboard usage values, see usb.org's HID-usage-tables document, chapter
 * 10 Keyboard/Keypad Page for more codes.
 */
#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

#define KEY_1       30
#define KEY_2       31
#define KEY_3       32
#define KEY_4       33
#define KEY_5       34
#define KEY_6       35
#define KEY_7       36
#define KEY_8       37
#define KEY_9       38
#define KEY_0       39
#define KEY_RETURN  40
#define KEY_MINUS   45
#define KEY_DOT     55

static void buildReport(void)
{
uchar   key = 0;

    if(nextDigit != NULL){
        key = *nextDigit;
    }
    reportBuffer[0] = 0;    /* no modifiers */
    reportBuffer[1] = key;
}


/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            buildReport();
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}

/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
 
// Modified by insideGadgets
// Improved OSCCAL search for ATtiny85 to search both 2 x 128 regions and not to exceed 8.8MHz as per 
// the warning on the ATtiny85 datasheet, page 32.
/*static void calibrateOscillator(void) {
	uchar step = 4;
	uchar trialValue = 124, optimumValue;
	int x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

	// Top 128 bits
	do{
		OSCCAL = trialValue + step;
		x = usbMeasureFrameLength(); // Proportional to current real frequency 
		if (x < targetValue) { // Frequency still too low 
			trialValue += step;
		}
		else { break; }
	}while (trialValue < 168);

	// Bottom 128 bits
	if (trialValue == 168) { // We know it couldn't find a good OSCCAL in top 128 bits
		trialValue = 92;
		do{
		  OSCCAL = trialValue + step;
		  x = usbMeasureFrameLength(); // Proportional to current real frequency 
		  if (x < targetValue) { // Frequency still too low 
				trialValue += step;
		  }
		  else { break; }
		}while (trialValue < 120);
	}

	// We have a precision of +/- 1 for optimum OSCCAL here now do a neighborhood search for optimum value
	optimumValue = trialValue;
	optimumDev = x; // This is certainly far away from optimum
	for (OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
		x = usbMeasureFrameLength() - targetValue;
		if (x < 0)
			x = -x;
		if (x < optimumDev){
			optimumDev = x;
			optimumValue = OSCCAL;
		}
	}
	OSCCAL = optimumValue;
}*/
/*
Note: This calibration algorithm may try OSCCAL values of up to 192 even if
the optimum value is far below 192. It may therefore exceed the allowed clock
frequency of the CPU in low voltage designs!
You may replace this search algorithm with any other algorithm you like if
you have additional constraints such as a maximum CPU clock.
For version 5.x RC oscillators (those with a split range of 2x128 steps, e.g.
ATTiny25, ATTiny45, ATTiny85), it may be useful to search for the optimum in
both regions.
*/

void    usbEventResetReady(void)
{
//	calibrateOscillator();
}
// End V-USB default required code

// Start USB communications
void startUSB(void) {
	usbDeviceDisconnect();
	_delay_ms(300); // 300 ms disconnect
	usbDeviceConnect();
	usbInit();
	
	// Send empty data to the PC as it seems like this needs to be done a few times before we can send real data, otherwise
	// some of our data won't be shown. This adds a 3-4 second delay but _is_ needed.
	for (int emptyData = 0; emptyData < 700; emptyData++) {
		usbPoll();
		if(usbInterruptIsReady()){ 
			reportBuffer[0] = 0;
			reportBuffer[1] = 0;
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
		}
		_delay_ms(5);
	}
}

